@@ -10,6 +10,7 @@ import shutil
 import sys
 import struct
 import subprocess
+import zlib
 import click
 from tqdm import tqdm
 from ips_util import Patch
@@ -262,9 +263,17 @@ class Stream(object):
 @click.option("--log", is_flag=True, default=False)
 def cli(log):
     if log:
-        logging.basicConfig(handlers=[logging.FileHandler(filename="tool.log", encoding="utf-8", mode="w")], format="[%(levelname)s] %(message)s", level=logging.DEBUG)
+        enableLogging()
     else:
-        logging.getLogger().disabled = True
+        disableLogging()
+
+
+def enableLogging():
+    logging.basicConfig(handlers=[logging.FileHandler(filename="tool.log", encoding="utf-8", mode="w")], format="[%(levelname)s] %(message)s", level=logging.DEBUG)
+
+
+def disableLogging():
+    logging.getLogger().disabled = True
 
 
 def logMessage(*messages):
@@ -303,7 +312,7 @@ def toHex(byte):
 
 def isAscii(s):
     for i in range(len(s)):
-        if ord(s[i]) >= 128:
+        if ord(s[i]) >= 128 or ord(s[i]) < 0x20:
             return False
     return True
 
@@ -765,6 +774,22 @@ def repackBinaryStrings(section, infile, outfile, binranges, freeranges=None, de
     return notfound
 
 
+def runCLI(command):
+    if len(sys.argv) > 1:
+        command()
+        return
+    with click.Context(command) as ctx:
+        click.echo(command.get_help(ctx))
+        click.echo("")
+    while True:
+        cmd = click.prompt('Type a command').strip()
+        cmdlow = cmd.lower()
+        if cmdlow == "exit" or cmdlow == "quit" or cmdlow == "q":
+            break
+        sys.argv = shlex.split(sys.argv[0] + " " + cmd)
+        command(standalone_mode=False)
+
+
 # Folders
 def makeFolder(folder, clear=True):
     if clear:
@@ -853,6 +878,12 @@ def execute(cmd, show=True):
             logDebug(result)
 
 
+def crcFile(f):
+    with open(f, "rb") as crcf:
+        crc = zlib.crc32(crcf.read())
+    return crc
+
+
 def xdeltaPatch(patchfile, infile, outfile):
     logMessage("Creating xdelta patch", patchfile, "...")
     xdelta = bundledExecutable("xdelta.exe")
